@@ -1,3 +1,25 @@
+/*
+ * Cheops Next Generation GUI
+ * 
+ * agent-nmapxml.c
+ * code to parse nmap XML output
+ *
+ * Copyright(C) 1999 Brent Priddy <toopriddy@mailcity.com>
+ * 
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *                                                                  
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111 USA
+ */
 #include "agent-nmapxml.h"
 
 #define DEBUG_NMAPXML
@@ -444,7 +466,7 @@ xmlDoc *runNmapScan(char *ip, unsigned long options, char *port_range)
 		strcat(cmd, " -p ");
 		strcat(cmd, port_range);
 	}
-	DEBUG(printf(__FUNCTION__ "(): scanning %s using '%s'\n", ip, cmd));
+	DEBUG(printf("%s(): scanning %s using '%s'\n", __FUNCTION__, ip, cmd));
 	pfp = popen(cmd, "r");
 	if(pfp == NULL)
 	{
