@@ -64,7 +64,9 @@
 #include "cerror.h"
 #include "ip_utils.h"
 
+#ifndef FREEBSD
 #include <shadow.h>
+#endif
 #include <pwd.h>
 
 char *crypt(const char *key, const char *salt);
@@ -217,6 +219,46 @@ static char *get_local_name()
 	return buf;
 }
 
+static int agent_read(agent *a, int fd, char *buffer, int length)
+{
+	int ret;
+
+#if defined(HAS_SSL) && defined(USING_SSL)
+	if(a->ssl)
+	{
+		while(1)
+		{
+			fprintf(stderr, "agent_read(): SSL_read()\n");
+			ret = SSL_read(a->ssl, buffer, length);
+			if(ret != 0)
+			{
+				if(ret < 0)
+				{
+					switch(SSL_get_error(a->ssl, ret))
+					{
+						case SSL_ERROR_WANT_READ:
+							fprintf(stderr, "agent_read(): SSL_ERROR_WANT_READ\n");
+							continue;
+							break;
+						case SSL_ERROR_WANT_WRITE:
+							fprintf(stderr, "agent_read(): SSL_ERROR_WANT_WRITE\n");
+							SSL_write(a->ssl, NULL, 0);
+							break;
+					}
+				}
+			}
+			break;
+		}
+	}
+	else
+#endif
+	{
+		ret = read(fd, buffer, length);
+	}
+	
+	return(ret);
+}
+
 static int handle_agent(int *id, int fd, short events, void *data)
 {
 	/*
@@ -271,7 +313,8 @@ static int handle_agent(int *id, int fd, short events, void *data)
 	// #  == this is the length of the message that we are to recieve
 	if(a->expected_length == -1)
 	{
-		res = read(fd, &h->len, res);
+//		res = read(fd, &h->len, res);
+		res = agent_read(a, fd, (char *)&h->len, res);
 		if(res < 0)
 		{
 			DEBUG(clog(LOG_WARNING,
@@ -303,7 +346,8 @@ static int handle_agent(int *id, int fd, short events, void *data)
 	}
 	else
 	{
-		res = read(fd, a->write_ptr, res);
+//		res = read(fd, a->write_ptr, res);
+		res = agent_read(a, fd, (char *)a->write_ptr, res);
 		if(res < 0)
 		{
 			DEBUG(clog(LOG_WARNING,
@@ -747,9 +791,6 @@ static int make_ipv4_socket(struct sockaddr_in *sin)
 	int flags;
 	int optval = 1;
 		
-	fprintf(stderr,"Starting agent on %s\n",ip2str(sin->sin_addr.s_addr));
-	
-	
 	if((s = socket(PF_INET, SOCK_STREAM, 0)) < 0)
 	{
 		clog(LOG_ERROR, "socket failed(%s)\n", strerror(errno));
@@ -815,10 +856,10 @@ void event_destroy_agent(agent *a)
 		return;
 	}
 
-	if( shutdown(a->s, 2) < 0)
+	if( shutdown(a->s, SHUT_WR) < 0)
 	{
 		DEBUG(
-			clog(LOG_WARNING, "shutdown(s,2) is not happy (%s)\n",strerror(errno));
+			clog(LOG_WARNING, "shutdown(s,SHUT_WR) is not happy (%s)\n",strerror(errno));
 		);
 	}
 	close(a->s);
@@ -878,53 +919,29 @@ int event_create_agent(int agent_type)
 	
 	if (agent_type & AGENT_TYPE_IPV4) 
 	{
-		if((gethostname(hostname,sizeof(hostname))) < 0)
-		{
-			clog(LOG_ERROR, "Unable to get local hostname\n");
-			res=-1;
-		}
-		else
-		{	
-			hp = gethostbyname(hostname);
-			if(!hp)
-			{
-				clog(LOG_ERROR, "Unable to resolve local hostname '%s'\n",hostname);
-			}
-			else
-			{
-				sin.sin_family = AF_INET;
-				sin.sin_port = htons(2300);
-#ifdef FreeBSD
-/*
- * XXX FIXME
- * not sure why this doesn't work in FBSD, something in the conversion goofes things up
- * ( I get a "Cannot bind to address" from bind() further down, but it actually is the
- * correct IP. I'm tired right now, so we'll accept everybody for the time being
- */
-				sin.sin_addr.s_addr = INADDR_ANY;
-#else
-				memcpy(&sin.sin_addr, hp->h_addr, sizeof(sin.sin_addr));
-#endif    
-	
-				res = make_ipv4_socket(&sin);
-				if (res > -1)
-				{
-					a = new_agent();
+		sin.sin_family = AF_INET;
+		sin.sin_port = htons(2300);
+		sin.sin_addr.s_addr = INADDR_ANY;
 
-					if (a) {
-						
-						a->type = AGENT_TYPE_IPV4;
-						a->s = res;
-						a->next = agents;
-						a->more = NULL;
-						a->ssl = NULL;
-						a->id = cheops_io_add(res, handle_ipv4_connect, CHEOPS_IO_IN, a);
-						agents = a;
-						res = 0;
-					} else {
-						res = -1;
-					}
-				}
+		res = make_ipv4_socket(&sin);
+		if (res > -1)
+		{
+			a = new_agent();
+
+			if (a) 
+			{
+				a->type = AGENT_TYPE_IPV4;
+				a->s = res;
+				a->next = agents;
+				a->more = NULL;
+				a->ssl = NULL;
+				a->id = cheops_io_add(res, handle_ipv4_connect, CHEOPS_IO_IN, a);
+				agents = a;
+				res = 0;
+			} 
+			else 
+			{
+				res = -1;
 			}
 		}
 	}
@@ -937,7 +954,8 @@ static inline agent *chk_agent(agent *a)
  	 * Check to be sure this agent really exists
 	 */
 	agent *ta = agents;
-	while(ta) {
+	while(ta) 
+	{
 		if (ta == a)
 			break;
 		ta = ta->next;
@@ -945,6 +963,56 @@ static inline agent *chk_agent(agent *a)
 	return ta;
 }
 
+int agent_write(agent *a, char *buffer, int length)
+{
+	int ret = -1;
+	
+	switch(a->type)
+	{
+		case AGENT_TYPE_LOCAL:
+			ret = write(a->s, buffer, length);
+			break;
+		case AGENT_TYPE_IPV4:
+#if defined(HAS_SSL) && defined(USING_SSL)
+			if(a->ssl)
+			{
+				while(1)
+				{
+					fprintf(stderr, "agent_write(): SSL_write()\n");
+					ret = SSL_write(a->ssl, buffer, length);
+					if(ret != 0)
+					{
+						if(ret < 0)
+						{
+							switch(SSL_get_error(a->ssl, ret))
+							{
+								case SSL_ERROR_WANT_READ:
+									fprintf(stderr, "agent_write(): SSL_ERROR_WANT_READ\n");
+									SSL_read(a->ssl, NULL, 0);
+									break;
+								case SSL_ERROR_WANT_WRITE:
+									fprintf(stderr, "agent_write(): SSL_ERROR_WANT_WRITE\n");
+									continue;
+									break;
+							}
+						}
+					}
+					break;
+				}
+			}
+			else
+#endif
+			{
+				ret = send(a->s, buffer, length,0);
+			}
+			break;
+		default:
+			ret=-1;
+			break;
+	}
+	return(ret);
+}
+
 int event_send(agent *a, event_hdr *h)
 {
 	/*
@@ -965,18 +1033,7 @@ int event_send(agent *a, event_hdr *h)
 	
 	a = chk_agent(a);
 	if (a) {
-		switch(a->type)
-		{
-			case AGENT_TYPE_LOCAL:
-				res = write(a->s, h, ntohl(h->len));
-				break;
-			case AGENT_TYPE_IPV4:
-				res = send(a->s, h, ntohl(h->len),0);
-				break;
-			default:
-				res=-1;
-				break;
-		}
+		res = agent_write(a, (char *)h, ntohl(h->len));
 		
 		if (res < 0) {
 			clog(LOG_WARNING, "write failed: %s\n", strerror(errno));
@@ -1075,6 +1132,7 @@ int send_auth_request_reply(agent *a, int authenticated)
 
 int handle_auth_request(event_hdr *h, event *e, agent *a)
 {
+#ifndef FREEBSD
 	char *user = e->auth_request_e.username;
 	char *pass = e->auth_request_e.password;
 	char *enc;
@@ -1116,9 +1174,17 @@ int handle_auth_request(event_hdr *h, event *e, agent *a)
 		authenticated = 0;
 	}
 	send_auth_request_reply(a, authenticated);
+#endif
 	return(1);
 }
 
+void event_cleanup(void)
+{
+	while(agents) 
+	{
+		event_destroy_agent(agents);
+	}
+}
+
 
 
-
