@@ -40,6 +40,7 @@
 #include "gui-pixmap.h"
 #include "gui-handlers.h"
 #include "gui-dns.h"
+#include "script.h"
 #ifdef USING_MONITORING
 	#include "gui-monitoring.h"
 #endif
@@ -65,6 +66,7 @@ typedef struct _poo_info {
 	GtkWidget   *menu;
 	page_object *po;
 	net_page    *np;
+	int          count;
 } poo_info;
 
 void set_widget_tooltips(GtkWidget *w, char *text);
@@ -171,7 +173,13 @@ page_object *page_object_new(GnomeCanvas *canvas)
 
 char *page_object_get_name(page_object *po)
 {
+	static char *hold = NULL;
 	char *name;
+	
+	if(hold)
+		g_free(hold);
+	hold = NULL;
+	
 	if(po)
 	{
 		if(po->name == NULL)
@@ -181,7 +189,21 @@ char *page_object_get_name(page_object *po)
 			if(options_use_ip_for_label)
 				name = ip2str(po->ip);
 			else
-				name = po->name;
+			{
+				if(options_only_display_hostname)
+				{
+					hold = g_strdup(po->name);
+					if( (name = strchr(hold, '.')) )
+					{
+						*name = '\0';
+					}
+					name = hold;
+				}
+				else
+				{
+					name = po->name;
+				}
+			}
 		}
 	}
 	else
@@ -199,7 +221,7 @@ void page_object_display( page_object *po )
 	double nexty = INITIAL_Y;
 	double height, width;
 	double nextydown = 0.0;
-	double gap_x, gap_y;
+	double tempx, tempy, tempwidth, tempheight;
 	char *name;
 
 	g_return_if_fail( po );
@@ -236,21 +258,18 @@ void page_object_display( page_object *po )
  */
 	name = page_object_get_name(po);
 	gnome_canvas_item_set(GNOME_CANVAS_ITEM(po->label), "text", name, NULL);
-	gnome_canvas_item_get_bounds(GNOME_CANVAS_ITEM(po->label), 
-	                             &x1, &y1, &x2, &y2);
 
-	gnome_canvas_item_i2w(GNOME_CANVAS_ITEM(po->label), &x1, &y1);
-	gnome_canvas_item_i2w(GNOME_CANVAS_ITEM(po->label), &x2, &y2);
-
-	gap_x = 2.0;
-	gap_y = 2.0;
-	gnome_canvas_item_i2w(GNOME_CANVAS_ITEM(po->label), &gap_x, &gap_y);
-//	printf("gap %f %f\n");
+	gtk_object_get(GTK_OBJECT(po->label), "x", &tempx,
+	                                      "y", &tempy,
+	                                      "text_height", &tempheight,
+	                                      "text_width", &tempwidth,
+	                                      NULL);
+	
 	gnome_canvas_item_set(GNOME_CANVAS_ITEM(po->box), 
-	                      "x1", x1-2.0, 
-	                      "x2", x2+2.0, 
-	                      "y1", y1-2.0, 
-	                      "y2", y2+2.0, NULL);
+	                      "x1", (tempx - tempwidth/2.0)-2.0, 
+	                      "x2", (tempx + tempwidth/2.0)+2.0, 
+	                      "y1", (tempy)-2.0, 
+	                      "y2", (tempy + tempheight)+2.0, NULL);
 
 /*
  * get rid of the image, cause we are going to redisplay it
@@ -517,6 +536,7 @@ void net_page_set_pixels_per_unit(net_page *np, double ppi)
 	{
 		np->pixels_per_unit = ppi;
 		gnome_canvas_set_pixels_per_unit(GNOME_CANVAS(np->canvas), np->pixels_per_unit);
+
 		gnome_canvas_world_to_window(GNOME_CANVAS(np->canvas),
 		                             CANVAS_FIXED_WIDTH, CANVAS_FIXED_HEIGHT,
 		                             &x, &y);
@@ -1812,16 +1832,17 @@ void page_object_show_properties(GtkWidget *w, gpointer data)
 		gtk_widget_show(hbox);
 		gtk_notebook_append_page(GTK_NOTEBOOK(main_notebook),hbox, label); 
 
-		vbox = gtk_vbox_new(FALSE, 5);
-		gtk_widget_show(vbox);
-		gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, FALSE, 5);
-
 #ifdef USING_MONITORING		
 		label = gtk_label_new("Monitoring");
 		gtk_widget_show(label);
 		gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 5);
-		po->monitoring_vbox = vbox;
-		po->monitoring_hbox = hbox;
+
+		vbox = gtk_vbox_new(FALSE, 5);
+		gtk_widget_show(vbox);
+		gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, FALSE, 5);
+
+		po->monitoring_vbox = hbox;
+		po->monitoring_fields_vbox = vbox;
 		gui_monitoring_toplevel(po);
 #else /* USING_MONITORING */
 		label = gtk_label_new("Not Working Yet");
@@ -1859,8 +1880,16 @@ void page_object_service_callback(GtkWidget *widget, gpointer data)
 	int protocol = (u32)data & 0xFF;
 
 	// there can only be one page_object selected that can show the service list,
-	// so i use this quick and dirty way to get at the IP address :)
-	service_callback( (get_current_net_page())->plink->ip,port, protocol);
+	// so i use this quick and dirty way to get at the page object :)
+	service_callback( (get_current_net_page())->plink, port, protocol);
+}
+
+void page_object_script_callback(GtkWidget *widget, script_t *script)
+{
+	page_object *po;
+	
+	for(po = get_current_net_page()->plink; po; po = po->plink)
+		run_command_callback(po, 0, 0, script->script);
 }
 
 int net_page_count_plink(net_page *np)
@@ -2117,6 +2146,27 @@ void page_object_option_list_addall(page_object_option *poo, poo_info *pooi)
 		poo->callback(pooi->np, pooi->po, pooi->menu, poo->data);
 }
 
+void page_object_script_list_add(script_t *script, poo_info *pooi)
+{
+	if(script && (script->flags & SCRIPT_FLAGS_VIEWABLE))
+	{
+		GtkWidget *item = gtk_menu_item_new_with_label(script->name);
+		gtk_signal_connect(GTK_OBJECT(item), "activate",
+		                   GTK_SIGNAL_FUNC(page_object_script_callback),
+		                   (gpointer)script);
+
+		if(net_page_count_plink(pooi->np) == 1)
+			set_widget_tooltips(item, "Run the script in Edit->Settings->Scripts on this host (from cheops-ng)");
+		else
+			set_widget_tooltips(item, "Run the script in Edit->Settings->Scripts on these hosts (from cheops-ng)");
+		
+		gtk_menu_append(GTK_MENU(pooi->menu), item);
+		gtk_widget_show(item);
+
+		pooi->count++;
+	}
+}
+
 /*
  * Shows an option list for things to do on a oage_object
  */
@@ -2143,6 +2193,17 @@ void page_object_show_option_list(net_page *np, page_object *po, GdkEventButton
 
 	os_seperator_item = gtk_menu_item_new();
 	gtk_menu_append(GTK_MENU(pagemenu), os_seperator_item);
+
+	pooi.count = 0;	
+	script_foreach((GFunc)page_object_script_list_add, &pooi);
+
+	if(pooi.count)
+	{
+		gtk_widget_show(os_seperator_item);
+
+		os_seperator_item = gtk_menu_item_new();
+		gtk_menu_append(GTK_MENU(pagemenu), os_seperator_item);
+	}
 	
 	if( net_page_count_plink(np) == 1 )
 	{
@@ -2468,15 +2529,23 @@ int page_object_event_dispatch(GtkObject *widget, GdkEvent *event, page_object *
 					
 				for(pl = np->plink; pl; pl = pl->plink)
 				{
+					double boundswidth=CANVAS_FIXED_WIDTH, boundsheight=CANVAS_FIXED_HEIGHT;
+					
+#if 0
+					gnome_canvas_window_to_world(GNOME_CANVAS(np->canvas),
+					                             CANVAS_FIXED_WIDTH, CANVAS_FIXED_HEIGHT,
+					                             &boundswidth, &boundsheight);
+					printf("%f %f\n", boundswidth, boundsheight);
+#endif
 					xx = x;
 					yy = y;
 					
-					if( ((pl->x1 + x) >= 0) && ((pl->x2 + x) <= CANVAS_FIXED_WIDTH) )
+					if( ((pl->x1 + x) >= 0) && ((pl->x2 + x) <= boundswidth) )
 						pl->x += x;
 					else
 						xx = 0;
 						
-					if( ((pl->y1 + y) >= 0) && ((pl->y2 + y) <= CANVAS_FIXED_HEIGHT) )
+					if( ((pl->y1 + y) >= 0) && ((pl->y2 + y) <= boundsheight) )
 						pl->y += y;
 					else
 						yy = 0;	
