@@ -65,7 +65,7 @@ USE_PTHREAD(pthread_mutex_t probe_mutex);
 
 /* Probing routines, may be useful for monitoring as well */
 
-GList *probes;
+GList *probes = NULL;
 
 
 static struct probe probesa[] = {
